#include <array>
#include <iostream>
#include <vector>
#include <deque>
#include <string>
#include <functional>
#include <iterator> // e.g. ostream_iterator
#include <algorithm> // e.g. unique

using namespace std;



inline bool justEqual(int al, int ar) {
   return al == ar;
}

/* Im Bereich zwischen anf und end werden alle doppelten
   Elemente ueberschrieben. Doppelte bedeutet,
   dass die Funktion f auf zwei Bereichselemente
   angewandt true liefert.
   Rueckgabewert ist ein Iterator, der ein Element
   hinter das neue Ende verweist.*/
template <typename Iter, typename Func>
Iter myUnique(Iter anf, Iter end, Func f = justEqual) {
   while (anf != end) {
      // alle folgenden doppelten Vorkommen von anf entfernen
      Iter iter = anf;
      ++iter;
      Iter nxtInsPos = iter;
      while (iter != end) {
         if (false == f(*anf, *iter)) {
            // keine Doublette, wir uebernehmen
            *nxtInsPos = *iter;
            ++nxtInsPos;
         }// if (false == f(*anf, *iter)) {
         ++iter;
      }//  while (iter != end) {
      if (end == nxtInsPos) {
         // no doubles found, therefore check nxt element
         ++anf;
      }
      else {
         // we found *anf more than once, no update of anf allowed
         end = nxtInsPos;
         ++anf;
      }
   }//    while (anf != end) 
   return end;
}


template <typename Iter, typename Str>
void PrintTo(ostream& str, Iter beg, Iter end, Str astr)
{
   str << astr;
   copy(beg, end, ostream_iterator<int>(str, ";"));
}




template <typename Tcont>
void PrintBeforeUniqueAfter(Tcont& cont)
{
   PrintTo(cout, cont.begin(), cont.end(), "before: ");
   auto newEnd = myUnique(cont.begin(), cont.end(), justEqual);
//   auto newEnd =unique(cont.begin(), cont.end(), justEqual);
   cout << endl;
   PrintTo(cout, cont.begin(), newEnd, "after:  ");
   cout << endl;
}


void uniqueWithManyContainers()
{
   array<int, 7> arr = { 1, 3, 3, 1, 5, 3, 9 };
   PrintBeforeUniqueAfter(arr);

   vector<int> vec = { 1, 3, 3, 1, 5, 3, 9 };
   PrintBeforeUniqueAfter(vec);

   deque<int> dequeInt = { 1, 3, 3, 1, 5, 3, 9 };
   PrintBeforeUniqueAfter(dequeInt);

}


void uniqueWithManyContainers2()
{
   array<int, 8> arr = { 1, 3, 3, 3, 5, 5, 1, 9 };
   PrintBeforeUniqueAfter(arr);

   vector<int> vec = { 1, 3, 3, 3, 5, 5, 1, 9 };
   PrintBeforeUniqueAfter(vec);

   deque<int> dequeInt= { 1, 3, 3, 3, 5, 5, 1, 9 };
   PrintBeforeUniqueAfter(dequeInt);

}


/* Implementieren Sie  den Algorithmus unique selbst
   mit Namen myUnique und weisen Sie nach, dass er das 
   gleiche Verhalten wie unique hat.
   Verwenden Sie dazu den Code aus Aufgabe 2
   nur eben, dass statt unique myUnique aufgerufen wird.
   Erweitern Sie myUnique, sodass nun wirklich alle
   Dubletten und nicht nur aufeinander folgende entfernt
   werden.
   */
void aufgabe3()
{
   cout << "Aufgabe3:\n";
   uniqueWithManyContainers();
   uniqueWithManyContainers2();
}

