#include <map>
#include <unordered_map>
#include <iterator>
#include <iomanip> // wegen setw
#include <utility> // wg. pair
#include <string>
#include<iostream>
using namespace std;





void FillElems(map<string, double >& ar_cont) {
   typedef map<string, double >::value_type elem;
   ar_cont.insert(elem("Meier", 8400.14));
   ar_cont.insert(elem("Schulz", 4600.26));
   ar_cont.insert(elem("Kohl", 8800.67));
   ar_cont.insert(elem("Eser", 147.73));
   // es wird nicht ueberschrieben, wenn schon vorhanden
   ar_cont.insert(elem("Meier", 2200.11));
   ar_cont.insert(elem("Kohl", 7880.67));
}

double CalcDurch(const map<string, double >& ar_cont){
   double durch = 0.0;
   for (const auto& iter : ar_cont) {
      durch += iter.second;
   }
   return ar_cont.empty() ? 0.0 : durch / ar_cont.size();

}
void PrintCont(
   const map<string, double >& ar_cont,
   double ad_durch
) {
   for (const auto& iter : ar_cont)   {
      cout << "Key: " << setw(7) << iter.first << ' '
         << "Value: " << setw(8) << iter.second
         << setw(6)
         << (iter.second > ad_durch ? " reich" : "arm")
         << endl;
   }
}


void FillElems(unordered_map<string, double >& ar_cont)
{
   typedef unordered_map<string, double >::value_type elem;
   ar_cont.insert(elem("Meier", 8400.14));
   ar_cont.insert(elem("Schulz", 4600.26));
   ar_cont.insert(elem("Kohl", 8800.67));
   ar_cont.insert(elem("Eser", 147.73));
   // es wird nicht ueberschrieben, wenn schon vorhanden
   ar_cont.insert(elem("Meier", 2200.11));
   ar_cont.insert(elem("Kohl", 7880.67));
}

double CalcDurch(const unordered_map<string, double >& ar_cont)
{
   double durch = 0.0;
   for (const auto& iter : ar_cont) {
      durch += iter.second;
   }
   return ar_cont.empty() ? 0.0 : durch / ar_cont.size();

}

void PrintCont
(
   const unordered_map<string, double >& ar_cont,
   double ad_durch
)
{

   // Ausgabe in der Sortierreihenfolge
   for (const auto& iter : ar_cont) 
   {
      cout << "Key: " << setw(7) << iter.first << ' '
         << "Value: " << setw(8) << iter.second
         << setw(6) 
         << (iter.second > ad_durch ? " reich" : "arm")
         << endl;
   }
}


void berechneDurchgehalt1() {
   map<string, double >  cont;
   FillElems(cont);
   PrintCont(cont, CalcDurch(cont));
}


void berechneDurchgehalt2()
{
   // Datentyp des Containers:
   typedef unordered_map<string, double >
      StringFloatMap;
   StringFloatMap cont;
   FillElems(cont);
   PrintCont(cont, CalcDurch(cont));
}


/*********************************** Template   ***************/

template <typename Cont>
void FillElemsTemp(Cont& ar_cont)
{
   // typename garantiert, dass elem als Typ hergeleitet wird, was erforderlich ist, wenn
   // sofort uebersetzt wird, bevor bekannt ist, was fuer Typen uebengeben werden
   // Letzteres ist z.B. bei gcc Compilern unter Linux der Fall
   typedef typename Cont::value_type elem;
   ar_cont.insert(elem("Meier", 8400.14));
   ar_cont.insert(elem("Schulz", 4600.26));
   ar_cont.insert(elem("Kohl", 8800.67));
   ar_cont.insert(elem("Eser", 147.73));
   // es wird nicht ueberschrieben, wenn schon vorhanden
   ar_cont.insert(elem("Meier", 2200.11));
   ar_cont.insert(elem("Kohl",7880.67));
}

template <typename Cont>
double CalcDurchTemp(const Cont& ar_cont)
{
   double durch = 0.0;
   for (const auto& iter : ar_cont) {
      durch += iter.second;
   }
   return ar_cont.empty() ? 0.0 : durch / ar_cont.size();

}

template <typename Cont>
void PrintContTemp
(
   const Cont& ar_cont,
   double ad_durch
)
{

   // Ausgabe in der Sortierreihenfolge
   for (const auto& iter : ar_cont)
   {
      cout << "Key: " << setw(7) << iter.first << ' '
         << "Value: " << setw(8) << iter.second
         << setw(6)
         << (iter.second > ad_durch ? " reich" : "arm")
         << endl;
   }
}

void berechneDurchgehaltTempMap()
{
   map<string, double >  cont;
   FillElemsTemp(cont);
   PrintContTemp(cont, CalcDurchTemp(cont));
}


void berechneDurchgehaltTempOrdered()
{
   unordered_map<string, double >  cont;
   FillElemsTemp(cont);
   PrintContTemp(cont, CalcDurchTemp(cont));
}


void berechneDurchgehaltTempMultiMap()
{
   multimap<string, double >  cont;
   FillElemsTemp(cont);
   PrintContTemp(cont, CalcDurchTemp(cont));
}


void berechneDurchgehaltTempMultiOrdered()
{
   unordered_multimap<string, double >  cont;
   FillElemsTemp(cont);
   PrintContTemp(cont, CalcDurchTemp(cont));
}

/*****************************************************************/
void aufgabe5()
{
   cout << "\nAufgabe5:\n   Mit map:\n";
   berechneDurchgehalt1();
   cout << "\n  Mit ordered_map:\n";
   berechneDurchgehalt2();

   cout << "\n   Mit map als Temp:\n";
   berechneDurchgehaltTempMap();
   cout << "\n  Mit ordered_map als Temp:\n";
   berechneDurchgehaltTempOrdered();

   cout << "\n   Mit map als Temp:\n";
   berechneDurchgehaltTempMultiMap();
   cout << "\n  Mit ordered_map als Temp:\n";
   berechneDurchgehaltTempMultiOrdered();
}

